/* /////////////////////////////////////////////////////////////////////////////
 * File:		perf.d
 *
 * Purpose:		Win32 performance types
 *
 * Created		15th March 2003
 * Updated:		16th September 2003
 *
 * Copyright:	Synesis Software Pty Ltd, (c) 2002. All rights reserved.
 *
 * Home:		http://www.synesis.com.au/software
 *
 * ////////////////////////////////////////////////////////////////////////// */



/** \file synsoft/win32/perf.d This file contains Win32 performance classes */

/* ////////////////////////////////////////////////////////////////////////// */

module synsoft.win32.perf;

/* ////////////////////////////////////////////////////////////////////////// */

import windows;

/* ////////////////////////////////////////////////////////////////////////// */

/// \defgroup group_synsoft_win32_perf synsoft.win32.perf
/// \ingroup group_synsoft_win32
/// \brief This library provides Win32 Performance facilities

/* ////////////////////////////////////////////////////////////////////////// */

/// \brief Performance counter scope class generating template
///
/// This template declares the class Scope which manages a performance counter 
/// instance, scoping its measurement interval by virtue of its supporting the
/// RAII protocol.
///
/// \ingroup group_synsoft_win32_perf
template PerformanceCounterScope(T)
{
	/// \brief Performance counter scoping class
	///
	/// This auto class manages a performance counter instance, scoping its
	/// measurement interval by virtue of its supporting the RAII protocol
	auto class Scope
	{
	private:
		alias Scope     class_type;
	public:
		/// The parameterising performance counter class
		alias T         counter_type;

	public:
		/// Constructs an instance of the PerformanceCounterScope using a reference 
		/// to the parameterising performance counter class, whose measurement is
		/// to be scoped. The measurement interval is commenced with a call to 
		/// <b>start()</b> on the managed counter instance.
		this(counter_type counter);

		/// The measurement interval is completed with a call to <b>stop()</b> on the 
		/// managed counter instance.
		~this();

		/// Calls <b>stop()</b> on the managed counter instance, so that intermediate
		/// timings can be taken.
		void Stop();

		/// Returns a reference to the managed counter instance.
		counter_type propget_Counter();

	// Members
	private:

	// Not to be implemented
	private:
		this(class_type rhs);
	}
}

/* ////////////////////////////////////////////////////////////////////////// */

/// \brief A performance counter that uses the high performance hardware counter on the host machine
///
/// This class provides high-resolution performance monitoring using the host machine's high performance
/// hardware counter. This class does not provide meaningful timing information on operating systems
/// that do not provide a high performance hardware counter.
///
/// \ingroup group_synsoft_win32_perf
class HighPerformanceCounter
{
private:
	alias	long	epoch_type;
public:
	/// \brief The interval type
	///
	/// The type of the interval measurement, a 64-bit signed integer
	alias	long	interval_type;

	/// \brief The scope type
	///
	/// The type with which instances of the counter class can be subject to the RAII protocol in order
	/// to scope particular measurement intervals
	alias instance PerformanceCounterScope(HighPerformanceCounter).Scope	scope_type;

// Constructors
private:
	static this();

// Operations
public:
	/// \brief Starts measurement
	///
	/// Begins the measurement period
    void Start();

	/// \brief Ends measurement
	///
	/// Ends the measurement period
    void Stop();

// Attributes
public:
	/// \brief The elapsed count in the measurement period
	///
	/// This represents the extent, in machine-specific increments, of the measurement period
    interval_type propget_PeriodCount();

	/// \brief The number of whole seconds in the measurement period
	///
	/// This represents the extent, in whole seconds, of the measurement period
    interval_type propget_Seconds();

	/// \brief The number of whole milliseconds in the measurement period
	///
	/// This represents the extent, in whole milliseconds, of the measurement period
    interval_type propget_Milliseconds();

	/// \brief The number of whole microseconds in the measurement period
	///
	/// This represents the extent, in whole microseconds, of the measurement period
    interval_type propget_Microseconds();

// Members
private:
}

/* ////////////////////////////////////////////////////////////////////////// */

/// \brief A low-cost, low-resolution performance counter
///
/// This class provides low-resolution, but low-latency, performance monitoring
/// and is guaranteed to be meaningful on all operating systems.
///
/// \ingroup group_synsoft_win32_perf
class TickCounter
{
private:
	alias	long	epoch_type;
public:
	/// \brief The interval type
	///
	/// The type of the interval measurement, a 64-bit signed integer
	alias	long	interval_type;

	/// \brief The scope type
	///
	/// The type with which instances of the counter class can be subject to the RAII protocol in order
	/// to scope particular measurement intervals
	alias instance PerformanceCounterScope(TickCounter).Scope	scope_type;

// Constructors
public:

// Operations
public:
	/// \brief Starts measurement
	///
	/// Begins the measurement period
    void Start();

	/// \brief Ends measurement
	///
	/// Ends the measurement period
    void Stop();

// Attributes
public:
	/// \brief The elapsed count in the measurement period
	///
	/// This represents the extent, in machine-specific increments, of the measurement period
    interval_type propget_PeriodCount();

	/// \brief The number of whole seconds in the measurement period
	///
	/// This represents the extent, in whole seconds, of the measurement period
    interval_type propget_Seconds();

	/// \brief The number of whole milliseconds in the measurement period
	///
	/// This represents the extent, in whole milliseconds, of the measurement period
    interval_type propget_Milliseconds();

	/// \brief The number of whole microseconds in the measurement period
	///
	/// This represents the extent, in whole microseconds, of the measurement period
    interval_type propget_Microseconds();

// Members
private:
}

/* ////////////////////////////////////////////////////////////////////////// */

/// \brief A performance counter that provides thread-specific performance timings
///
/// This class uses the operating system's performance monitoring facilities to provide timing 
/// information pertaining to the calling thread only, irrespective of the activities of other
/// threads on the system. This class does not provide meaningful timing information on operating
/// systems that do not provide thread-specific monitoring.
///
/// \ingroup group_synsoft_win32_perf
class ThreadTimesCounter
{
private:
	alias	long	epoch_type;
public:
	/// \brief The interval type
	///
	/// The type of the interval measurement, a 64-bit signed integer
	alias	long	interval_type;

	/// \brief The scope type
	///
	/// The type with which instances of the counter class can be subject to the RAII protocol in order
	/// to scope particular measurement intervals
	alias instance PerformanceCounterScope(ThreadTimesCounter).Scope	scope_type;

// Constructors
public:
	/// \brief Constructor
	///
	/// Creates an instance of the class, and caches the thread token so that measurements will
	/// be taken with respect to the thread in which the class was created.
	this();

// Operations
public:
	/// \brief Starts measurement
	///
	/// Begins the measurement period
    void Start();

	/// \brief Ends measurement
	///
	/// Ends the measurement period
    void Stop();

// Attributes
public:

    // Kernel

	/// \brief The elapsed count in the measurement period for kernel mode activity
	///
	/// This represents the extent, in machine-specific increments, of the measurement period for kernel mode activity
    interval_type propget_KernelPeriodCount();

	/// \brief The number of whole seconds in the measurement period for kernel mode activity
	///
	/// This represents the extent, in whole seconds, of the measurement period for kernel mode activity
    interval_type propget_KernelSeconds();

	/// \brief The number of whole milliseconds in the measurement period for kernel mode activity
	///
	/// This represents the extent, in whole milliseconds, of the measurement period for kernel mode activity
    interval_type propget_KernelMilliseconds();

	/// \brief The number of whole microseconds in the measurement period for kernel mode activity
	///
	/// This represents the extent, in whole microseconds, of the measurement period for kernel mode activity
    interval_type propget_KernelMicroseconds();

    // User

	/// \brief The elapsed count in the measurement period for user mode activity
	///
	/// This represents the extent, in machine-specific increments, of the measurement period for user mode activity
    interval_type propget_UserPeriodCount();

	/// \brief The number of whole seconds in the measurement period for user mode activity
	///
	/// This represents the extent, in whole seconds, of the measurement period for user mode activity
    interval_type propget_UserSeconds();

	/// \brief The number of whole milliseconds in the measurement period for user mode activity
	///
	/// This represents the extent, in whole milliseconds, of the measurement period for user mode activity
    interval_type propget_UserMilliseconds();

	/// \brief The number of whole microseconds in the measurement period for user mode activity
	///
	/// This represents the extent, in whole microseconds, of the measurement period for user mode activity
    interval_type propget_UserMicroseconds();

    // Total

	/// \brief The elapsed count in the measurement period
	///
	/// This represents the extent, in machine-specific increments, of the measurement period
    interval_type propget_PeriodCount();

	/// \brief The number of whole seconds in the measurement period
	///
	/// This represents the extent, in whole seconds, of the measurement period
    interval_type propget_Seconds();

	/// \brief The number of whole milliseconds in the measurement period
	///
	/// This represents the extent, in whole milliseconds, of the measurement period
    interval_type propget_Milliseconds();

	/// \brief The number of whole microseconds in the measurement period
	///
	/// This represents the extent, in whole microseconds, of the measurement period
    interval_type propget_Microseconds();

// Members
private:
}

/* ////////////////////////////////////////////////////////////////////////// */

/// \brief A performance counter that provides process-specific performance timings
///
/// This class uses the operating system's performance monitoring facilities to provide timing 
/// information pertaining to the calling process only, irrespective of the activities of other
/// processes on the system. This class does not provide meaningful timing information on operating
/// systems that do not provide process-specific monitoring.
///
/// \ingroup group_synsoft_win32_perf
class ProcessTimesCounter
{
private:
	alias	long	epoch_type;
public:
	/// \brief The interval type
	///
	/// The type of the interval measurement, a 64-bit signed integer
	alias	long	interval_type;

	/// \brief The scope type
	///
	/// The type with which instances of the counter class can be subject to the RAII protocol in order
	/// to scope particular measurement intervals
	alias instance PerformanceCounterScope(ProcessTimesCounter).Scope	scope_type;

// Constructors
private:
	/// \brief Class constructor
	///
	/// Detects availability of the high performance hardware counter, and if
	/// not available adjusts 
	static this();

// Operations
public:
	/// \brief Starts measurement
	///
	/// Begins the measurement period
    void Start();

	/// \brief Ends measurement
	///
	/// Ends the measurement period
    void Stop();

// Attributes
public:

    // Kernel

	/// \brief The elapsed count in the measurement period for kernel mode activity
	///
	/// This represents the extent, in machine-specific increments, of the measurement period for kernel mode activity
    interval_type propget_KernelPeriodCount();

	/// \brief The number of whole seconds in the measurement period for kernel mode activity
	///
	/// This represents the extent, in whole seconds, of the measurement period for kernel mode activity
    interval_type propget_KernelSeconds();

	/// \brief The number of whole milliseconds in the measurement period for kernel mode activity
	///
	/// This represents the extent, in whole milliseconds, of the measurement period for kernel mode activity
    interval_type propget_KernelMilliseconds();

	/// \brief The number of whole microseconds in the measurement period for kernel mode activity
	///
	/// This represents the extent, in whole microseconds, of the measurement period for kernel mode activity
    interval_type propget_KernelMicroseconds();

    // User

	/// \brief The elapsed count in the measurement period for user mode activity
	///
	/// This represents the extent, in machine-specific increments, of the measurement period for user mode activity
    interval_type propget_UserPeriodCount();

	/// \brief The number of whole seconds in the measurement period for user mode activity
	///
	/// This represents the extent, in whole seconds, of the measurement period for user mode activity
    interval_type propget_UserSeconds();

	/// \brief The number of whole milliseconds in the measurement period for user mode activity
	///
	/// This represents the extent, in whole milliseconds, of the measurement period for user mode activity
    interval_type propget_UserMilliseconds();

	/// \brief The number of whole microseconds in the measurement period for user mode activity
	///
	/// This represents the extent, in whole microseconds, of the measurement period for user mode activity
    interval_type propget_UserMicroseconds();

    // Total

	/// \brief The elapsed count in the measurement period
	///
	/// This represents the extent, in machine-specific increments, of the measurement period
    interval_type propget_PeriodCount();

	/// \brief The number of whole seconds in the measurement period
	///
	/// This represents the extent, in whole seconds, of the measurement period
    interval_type propget_Seconds();

	/// \brief The number of whole milliseconds in the measurement period
	///
	/// This represents the extent, in whole milliseconds, of the measurement period
    interval_type propget_Milliseconds();

	/// \brief The number of whole microseconds in the measurement period
	///
	/// This represents the extent, in whole microseconds, of the measurement period
    interval_type propget_Microseconds();
}

/* ////////////////////////////////////////////////////////////////////////// */

/// \brief A performance counter that uses the most accurate measurement APIs available on the host machine
///
/// This class attempts to use the high performance hardware counter as its measurement resource, but failing 
/// that it defaults to less accurate resources in order to guarantee that meaningful measurements are always
/// available to application code
///
/// \ingroup group_synsoft_win32_perf
class PerformanceCounter
{
private:
	alias	long	epoch_type;
public:
	/// \brief The interval type
	///
	/// The type of the interval measurement, a 64-bit signed integer
	alias	long	interval_type;

	/// \brief The scope type
	///
	/// The type with which instances of the counter class can be subject to the RAII protocol in order
	/// to scope particular measurement intervals
	alias instance PerformanceCounterScope(PerformanceCounter).Scope	scope_type;

// Constructors
private:
	/// \brief Class constructor
	///
	/// Detects availability of the high performance hardware counter, and if
	/// not available adjusts 
	static this();

// Operations
public:
	/// \brief Starts measurement
	///
	/// Begins the measurement period
    void Start();

	/// \brief Ends measurement
	///
	/// Ends the measurement period
    void Stop();

// Attributes
public:
	/// \brief The elapsed count in the measurement period
	///
	/// This represents the extent, in machine-specific increments, of the measurement period
    interval_type propget_PeriodCount();

	/// \brief The number of whole seconds in the measurement period
	///
	/// This represents the extent, in whole seconds, of the measurement period
    interval_type propget_Seconds();

	/// \brief The number of whole milliseconds in the measurement period
	///
	/// This represents the extent, in whole milliseconds, of the measurement period
    interval_type propget_Milliseconds();

	/// \brief The number of whole microseconds in the measurement period
	///
	/// This represents the extent, in whole microseconds, of the measurement period
    interval_type propget_Microseconds();
}

/* ////////////////////////////////////////////////////////////////////////// */
