/* /////////////////////////////////////////////////////////////////////////////
 * File:		synch.d
 *
 * Purpose:		Win32 synchronisation types
 *
 * Created		15th March 2003
 * Updated:		16th September 2003
 *
 * Copyright:	Synesis Software Pty Ltd, (c) 2002. All rights reserved.
 *
 * Home:		http://www.synesis.com.au/software
 *
 * ////////////////////////////////////////////////////////////////////////// */



/** \file synsoft/win32/synch.d This file contains Win32 synchronisation classes */

/* ////////////////////////////////////////////////////////////////////////// */

module synsoft.win32.synch;

/* ////////////////////////////////////////////////////////////////////////// */

import synsoft.types;
import synsoft.win32.types;
import synsoft.win32.exception;
import string;

/* ////////////////////////////////////////////////////////////////////////// */

/// \defgroup group_synsoft_win32_synch synsoft.win32.synch
/// \ingroup group_synsoft_win32
/// \brief This library provides Win32 Synchronisation facilities

/* ////////////////////////////////////////////////////////////////////////// */

////////////////////////////////////////////////////////////////////////////////
// SynchException

/// This class is thrown by synchronisation classes in response to serious failures
///
/// \ingroup group_synsoft_win32_synch

class SynchException
	: Win32Exception
{
/// \name Construction
//@{
public:
	/// \brief Creates an instance of the exception
	///
	/// \param message The message associated with the exception
	this(char[] message);
	/// \brief Creates an instance of the exception, with the given 
	///
	/// \param message The message associated with the exception
	/// \param error The Win32 error number associated with the exception
	this(char[] message, int error);
//@}
}

////////////////////////////////////////////////////////////////////////////////
// Semaphore

/// This class represents a Win32 semaphore synchronisation object
///
/// \ingroup group_synsoft_win32_synch

class Semaphore
{
/// \name Constants
//@{
public:

	/// Represents an infinite wait period
	const uint INFINITE	=	(uint)0xFFFFFFFF;
//@}


/// \name Construction
//@{
public:

	/// \brief Creates an anonymous semaphore with the indicated number of slots. All slots are available
	///
	/// \param maxSlots The maximum number of slots
	this(int maxSlots);

	/// \brief Creates an anonymous semaphore with the indicated number of slots.
	///
	/// \param maxSlots The maximum number of slots
	/// \param availableSlots The number of slots initially available
	this(int maxSlots, int availableSlots);

	/// \brief Creates a named semaphore with the indicated number of slots. All slots are available
	///
	/// \param name The name of the semaphore
	/// \param maxSlots The maximum number of slots
	this(char[] name, int maxSlots);

	/// \brief Creates a named semaphore with the indicated number of slots.
	///
	/// \param name The name of the semaphore
	/// \param maxSlots The maximum number of slots
	/// \param availableSlots The number of slots initially available
	/// \note If the semaphore cannot be created the method throws a SynchException object
	this(char[] name, int maxSlots, int availableSlots);

	/// \brief Calls the close() method, ensuring that the semaphore is released
	/// to the operating system when the object is destroyed
	~this();
//@}

/// \name Methods
//@{
public:
	/// Closes the semaphore object handle
	void Close();

//	deprecated void close() { Close(); }

	/// \brief Waits, for the specified period, for a slot on the semaphore to become available
	///
	/// \param timeout The number of milliseconds to wait, or INFINITE for a non-expiring wait
	/// \return <b>true</b> if the semaphore slot was aquired, or <b>false</b> if the timeout period elapsed
	/// \note If the wait fails the method throws a SynchException object
	boolean Wait(uint timeout);

//	deprecated boolean wait(uint timeout) { return Wait(timeout); }

	/// \brief Waits forever for a slot on the semaphore to become available
	///
	/// \note If the wait fails the method throws a SynchException object
	void Wait();

//	deprecated void wait() { Wait(); }

	/// \brief Releases the given number of slots acquired on the semaphore.
	///
	/// \param cSlots The number of slots to release
	/// \note A SynchException object is thrown if the release fails
	void Release(uint cSlots);

//	deprecated void release(uint cSlots) { Release(cSlots); }

	/// \brief Releases one slot acquired on the semaphore.
	///
	/// \note A SynchException object is thrown if the release fails
	void Release();

//	deprecated void release() { Release(); }

//@}

/// \name Properties
//@{
public:

	/// The maximum number of slots in the semaphore
	int propget_NumSlots();

	/// The number of available slots in the semaphore
	int propget_AvailableSlots();

	/// The name of the semaphore
	char[] propget_Name();

//@}

}
